/**
  ******************************************************************************
  *
  *  Programed by Y.Ishioka
  *
  ******************************************************************************
  */
#if 0
#include "dev_head.h"
#else
#include "stm32f4xx.h"
#endif


/**
  ******************************************************************************
  */
void  dev_gpio_init( void )
{
  /* set clock enable */
  RCC->AHB1ENR |= ( RCC_AHB1ENR_GPIOAEN
                  | RCC_AHB1ENR_GPIOCEN
                  ) ;

  GPIOC->MODER   &= ~(0x03ffffff);
  GPIOC->MODER   |=  (0x00000000);
  GPIOC->OTYPER  |=  (0x1fff);
  GPIOC->OSPEEDR &= ~(0x03ffffff);
  GPIOC->OSPEEDR |=  (0x02aaaaaa);
  GPIOC->PUPDR   &= ~(0x03ffffff);
  GPIOC->AFR[0]  &= ~(0xffffffff);
  GPIOC->AFR[1]  &= ~(0x000fffff);

  /** for BTN:PC13 **/
  GPIOC->MODER   &= ~(0x0c000000);
  GPIOC->MODER   |=  (0x00000000);
  GPIOC->OTYPER  |=  (0x2000);
  GPIOC->PUPDR   &= ~(0x0c000000);
  GPIOC->PUPDR   |=  (0x04000000);
  GPIOC->AFR[0]  &= ~(0x00000000);
  GPIOC->AFR[1]  &= ~(0x00f00000);

  /** for TEST-OUTPUT:PA8 **/
  GPIOA->MODER   &= ~(0x00030000);
  GPIOA->MODER   |=  (0x00010000);
  GPIOA->OTYPER  &= ~(0x0100);
  GPIOA->OTYPER  |=  (0x0000);
  GPIOA->OSPEEDR &= ~(0x00030000);
  GPIOA->OSPEEDR |=  (0x00030000);
  GPIOA->PUPDR   &= ~(0x00030000);
  GPIOA->PUPDR   |=  (0x00020000);
  GPIOA->AFR[0]  &= ~(0x00000000);
  GPIOA->AFR[1]  &= ~(0x0000000f);
}


unsigned short int  dev_gpio_c_read( void )
{
  return  (unsigned short int)(GPIOC->IDR) & 0xffff;
}


void  dev_gpio_a8_out( int stat )
{
  if( stat == 0 ) {
    GPIOA->BSRR = 0x0100 << 16;
  } else {
    GPIOA->BSRR = 0x0100;
  }
}


int  dev_get_btn_stat( void )
{
  int  stat = 0 ;

  if( (GPIOC->IDR & (1 << 13)) == 0 ) {
    stat |= 0x01 ;
  }

  return( stat ) ;
}


/*
 * ADC initialize
 */
void  dev_adc_init( void )
{
  /* set clock enable */
  RCC->APB2ENR |= RCC_APB2ENR_ADC1EN ;
  RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN ;

  /** for ADC:PA0 **/
  GPIOA->MODER   &= ~(0x00000003);
  GPIOA->MODER   |=  (0x00000003); /* set Analog Mode */
  GPIOA->PUPDR   &= ~(0x00000003);

  ADC1->CR1  =   (0x00000000);
  ADC1->CR2  =   (0x00000001); /* ADC ON */
  ADC1->SQR3 &= ~(0x3fffffff);
  ADC1->SQR3 |=  (0<<0) ;
  ADC1->SQR1 &= ~(0x00ffffff);
  ADC1->SQR1 |=  (0<<20) ;
  ADC1->CR2  |=  (0x40000000); /* set SWSTART */
}


/*
 *  read ADC result data
 *    return : == 0xffffffff : fail or not completed
 */
unsigned int  dev_adc_read( void )
{
  unsigned int  result = 0;

  if( !(ADC1->SR & 0x00000002) ) {
    return  -1 ;
  }

  result = (unsigned int)ADC1->DR ;

  ADC1->CR2  |=  (0x40000000); /* set SWSTART */

  return( result ) ;
}


void  dev_timer_init( void )
{
  /* set clock enable */
  RCC->APB1ENR |= ( RCC_APB1ENR_TIM2EN
                  ) ;

  /* counter disable */
  TIM2->CR1 &= ~TIM_CR1_CEN;

  /* clk/4, Auto-reload preload enable, downcounter, Update request source */
  TIM2->CR1  =  TIM_CR1_CKD_1 | TIM_CR1_ARPE | TIM_CR1_DIR | TIM_CR1_URS;
  TIM2->DIER =  TIM_DIER_UIE;
  TIM2->EGR  =  TIM_EGR_UG;

  TIM2->PSC = 10-1;

  /*  45000/9000000 = 1/200   ->   5msec */
  /*   9000/9000000 = 1/1000  ->   1msec */
  /*    900/9000000 = 1/10000 -> 100usec */
  TIM2->ARR = 900;
  TIM2->CNT = 900;

  TIM2->SR &= ~TIM_SR_UIF;
  TIM2->SR &= ~(TIM_SR_CC1IF | TIM_SR_CC2IF | TIM_SR_CC3IF | TIM_SR_CC4IF);

  TIM2->CR1 |=  TIM_CR1_CEN;   /* counter enable */
}


unsigned short int  dev_timer_clr_int( void )
{
  TIM2->SR &= ~TIM_SR_UIF; /* clear update interrupt */
  //TIM2->SR &= ~(TIM_SR_CC1IF | TIM_SR_CC2IF | TIM_SR_CC3IF | TIM_SR_CC4IF);

  return  0;
}

unsigned short int  dev_timer_read( void )
{
  return  TIM2->CNT;
}

