/*
   out_pulse for STM32F4xx-nucleo

     Copyright (C) by Y.Ishioka
*/

#include <stm32f4xx.h>
#include "kernel.h"
#include "syssvc/syslog.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void  xspi_init( void );
unsigned short int  dev_gpio_c_read( void );
int  dev_get_btn_stat( void );
unsigned int  dev_adc_read( void );

void  lcd_init( void );
void  lcd_setblock( unsigned short int x0, unsigned short int x1, unsigned short int y0, unsigned short int y1 );
void  lcd_pset( int x, int y, int color );
void  lcd_pset_fonta( unsigned char *font, int x, int y, int fg, int bg );
void  lcd_pset_fontk( unsigned char *font, int x, int y, int fg, int bg );
void  lcd_pset_fontaR( unsigned char *font, int x, int y, int fg, int bg );
void  lcd_put_stringR( int x, int y, int fg, int bg, unsigned char *msg, int size );
void  lcd_put_string( int x, int y, int fg, int bg, unsigned char *msg, int size );
void  lcd_dispbuff( unsigned char *buff );
void  lcd_fill( int x0, int x1, int y0, int y1, int color );
void  lcd_fill_full( int color );

void  WaitUsec( int );

#define  RED          0xf800
#define  GREEN        0x07e0
#define  BLUE         0x001f
#define  BLACK        0x0000
#define  GLAY         0x39e7
#define  D_BLUE       0x000f

#define  SCAN_POINT   512
#define  SCAN_LINE    13
#define  DISP_MAX_X   240
#define  DISP_MAX_Y   261
#define  LINE_BASE_X  235
#define  LINE_SKIP    18
#define  LINE_WIDTH   12

#define  DEF_ACD_DIFF   100

int  colro_tbl[] = {0xffff, RED, GREEN, BLUE, RED|GREEN, RED|BLUE, GREEN|BLUE, GLAY };
int  color_cnt;

int  scan_time_wait;
int  scan_time_count;
int  scan_data_count;
unsigned short int  *scan_buff_pnt;
unsigned int  scan_adc_before;

unsigned char  disp_buff[ DISP_MAX_X * DISP_MAX_Y ];
unsigned short int  scan_buff[ SCAN_POINT ];


void  disp_time_wait( void )
{
  unsigned char  msg[3];
  int  time, ten;
  int  i;

  time = scan_time_wait;
  ten = 0;
  for( i=0 ; i<10 ; i++ ) {
    if( time < 10 ) {
      break;
    }
    ten++;
    time -= 10;
  }
  msg[0] = '0' + ten;
  msg[1] = '0' + time;
  msg[2] = 0x00;

  lcd_put_stringR( 235-15, 318, BLACK, GREEN|BLUE, (unsigned char *)msg, strlen( (void *)msg ) );
}


void  disp_change_base_time( void )
{
  unsigned int  scan;
  scan = dev_adc_read();
  if( scan < scan_adc_before-DEF_ACD_DIFF || scan_adc_before+DEF_ACD_DIFF < scan ) {
    scan_time_wait = (scan >> 8) + 1;
    scan_adc_before = scan;
    disp_time_wait();
  }
}


void  disp_init( void )
{
    xspi_init();
    lcd_init();

    color_cnt = 0;

    scan_time_wait = 10;
    scan_time_count = 0;
    scan_data_count = 99999;
    scan_buff_pnt = scan_buff;

    scan_adc_before = 999999;
}


void  disp_base( void )
{
    int  i;
    int  x;
    int  color;
    const char *msg[] = {
      "  IDLE",
      "Syslog",
      "TASK01",
      "TASK02",
      "TASK03",
      "TASK04",
      "TASK05",
      "TASK06",
      "TASK07",
      "TASK08",
      "TASK09",
      "TASK10",
      "TASK11",
    };

    color = 0xffff;
    for( i=0, x=235 ; i<13 ; i++, x-=18 ) {
      lcd_put_stringR( x-15, 318, color, D_BLUE, (unsigned char *)msg[i], strlen( msg[i] ) );
    }

    disp_time_wait();
}


int  scan_data_line( void )
{
  if( ++scan_time_count < scan_time_wait ) {
    return  0;
  }
  scan_time_count = 0;

  if( scan_data_count >= DISP_MAX_Y ) {
    scan_buff_pnt = scan_buff;
    scan_data_count = 0;
    return  1;
  }

  *scan_buff_pnt++ = dev_gpio_c_read();
  scan_data_count++;

  return  0;
}


void  disp_main( void )
{
  int  i, j, k;
  unsigned char  *buff, *crnt;
  unsigned short int  *scan;
  unsigned short int  bitpat;

  if( dev_get_btn_stat() ) {
    return;
  }

  /* clear display buffer */
  buff = disp_buff;
  for( i=0 ; i<(DISP_MAX_X*DISP_MAX_Y) ; i++ ) {
      *buff++ = 0x00;
  }

  scan = scan_buff;
  buff = disp_buff + (DISP_MAX_Y-1) * DISP_MAX_X;
  for( i=0 ; i<DISP_MAX_Y ; i++ ) {
    bitpat = 0x0001;
    crnt = buff + LINE_BASE_X;
    for( j=0 ; j<SCAN_LINE ; j++ ) {
      if( *scan & bitpat ) {
        for( k=0 ; k<LINE_WIDTH ; k++ ) {
          *crnt-- = 1;
        }
        crnt -= (LINE_SKIP - LINE_WIDTH);
      } else {
        *crnt = 1;
        crnt -= LINE_SKIP;
      }
      bitpat <<= 1;
    }
    buff -= DISP_MAX_X;
    scan++;
  }

  lcd_dispbuff( disp_buff );
}


